=== modified file 'network-hooks.d/bridge'
--- network-hooks.d/bridge	2011-12-27 03:56:39 +0000
+++ network-hooks.d/bridge	2011-12-31 14:05:57 +0000
@@ -15,13 +15,18 @@
 
 CONFIG="$MANDOSNETHOOKDIR/bridge.conf"
 
-# Read config file, which must set "BRIDGE", "PORTS", and optionally
-# "IPADDRS" and "ROUTES".
+addrtoif(){
+    grep -liFe "$1" /sys/class/net/*/address \
+	| sed -e 's,.*/\([^/]*\)/[^/]*,\1,'
+}
+
+# Read config file, which must set "BRIDGE", "PORT_ADDRESSES", and
+# optionally "IPADDRS" and "ROUTES".
 if [ -e "$CONFIG" ]; then
     . "$CONFIG"
 fi
 
-if [ -z "$BRIDGE" -o -z "$PORTS" ]; then
+if [ -z "$BRIDGE" -o -z "$PORT_ADDRESSES" ]; then
     exit
 fi
 
@@ -39,9 +44,10 @@
 case "$1" in
     start)
 	"$brctl" addbr "$BRIDGE"
-	for port in $PORTS; do
-	    "$brctl" addif "$BRIDGE" "$port"
-	    ip link set dev "$port" up
+	for address in $PORT_ADDRESSES; do
+	    interface=`addrtoif "$address"`
+	    "$brctl" addif "$BRIDGE" "$interface"
+	    ip link set dev "$interface" up
 	done
 	ip link set dev "$BRIDGE" up
 	sleep "$DELAY"
@@ -58,9 +64,10 @@
 	;;
     stop)
 	ip link set dev "$BRIDGE" down
-	for port in $PORTS; do
-	    ip link set dev "$port" down
-	    "$brctl" delif "$BRIDGE" "$port"
+	for address in $PORT_ADDRESSES; do
+	    interface=`addrtoif "$address"`
+	    ip link set dev "$interface" down
+	    "$brctl" delif "$BRIDGE" "$interface"
 	done
 	"$brctl" delbr "$BRIDGE"
 	;;

=== modified file 'network-hooks.d/bridge.conf'
--- network-hooks.d/bridge.conf	2011-11-24 20:15:24 +0000
+++ network-hooks.d/bridge.conf	2011-12-31 13:25:58 +0000
@@ -2,7 +2,7 @@
 
 #BRIDGE=br0
 
-#PORTS="eth0 eth1"
+#PORT_ADDRESSES="00:11:22:33:44:55 11:22:33:44:55:66"
 
 ## Optional
 

=== modified file 'network-hooks.d/openvpn'
--- network-hooks.d/openvpn	2011-12-02 16:52:50 +0000
+++ network-hooks.d/openvpn	2011-12-31 14:05:57 +0000
@@ -16,7 +16,9 @@
 CONFIG="openvpn.conf"
 
 # Extract the "dev" setting from the config file
-VPNDEVICE="`sed -n -e 's/#.*//' -e 's/^[[:space:]]*dev[[:space:]]\+//p' \"$MANDOSNETHOOKDIR/$CONFIG\"`"
+VPNDEVICE=`sed -n -e 's/[[:space:]]#.*//' \
+    -e 's/^[[:space:]]*dev[[:space:]]\+//p' \
+    "$MANDOSNETHOOKDIR/$CONFIG"`
 
 PIDFILE=/run/openvpn-mandos.pid
 
@@ -34,7 +36,9 @@
 
 case "$1" in
     start)
-	"$openvpn" --cd "$MANDOSNETHOOKDIR" --daemon 'openvpn(Mandos)' --writepid "$PIDFILE" --config "$CONFIG"
+	"$openvpn" --cd "$MANDOSNETHOOKDIR" \
+	    --daemon 'openvpn(Mandos)' --writepid "$PIDFILE" \
+	    --config "$CONFIG"
 	sleep "$DELAY"
 	;;
     stop)

=== modified file 'network-hooks.d/wireless'
--- network-hooks.d/wireless	2011-12-30 17:14:15 +0000
+++ network-hooks.d/wireless	2011-12-31 14:05:57 +0000
@@ -20,6 +20,11 @@
 
 CONFIG="$MANDOSNETHOOKDIR/wireless.conf"
 
+addrtoif(){
+    grep -liFe "$1" /sys/class/net/*/address \
+	| sed -e 's,.*/\([^/]*\)/[^/]*,\1,'
+}
+
 # Read config file
 if [ -e "$CONFIG" ]; then
     . "$CONFIG"
@@ -27,13 +32,16 @@
     exit
 fi
 
-interfaces="`env|sed -n -e 's/^\(MODULE\|IPADDRS\|ROUTES\|WPA_DRIVER\)_\([^=]*\)=.*/\2/p' \"$CONFIG\" |sort -u`"
+ifkeys=`env | sed -n -e 's/^ADDRESS_\([^=]*\)=.*/\1/p' "$CONFIG" \
+    | sort -u`
 
 # Exit if DEVICE is set and is not any of the wireless interfaces
 if [ -n "$DEVICE" ]; then
     while :; do
-	for IF in $interfaces; do
-	    if [ "$IF" = "$DEVICE" ]; then
+	for KEY in $ifkeys; do
+	    ADDRESS=`eval 'echo "$ADDRESS_'"$KEY"\"`
+	    INTERFACE=`addrtoif "$ADDRESS"`
+	    if [ "$INTERFACE" = "$DEVICE" ]; then
 		break 2
 	    fi
 	done
@@ -66,29 +74,40 @@
     start)
 	mkdir -m u=rwx,go= -p "$CTRLDIR"
 	"$wpa_supplicant" -B -g "$CTRL" -p "$CTRLDIR" $WPAS_OPTIONS
-        for INTERFACE in $interfaces; do
-	    DRIVER=`eval 'echo "$WPA_DRIVER_'"$INTERFACE"\"`
-	    DELAY=`eval 'echo "$DELAY_'"$INTERFACE"\"`
+        for KEY in $ifkeys; do
+	    ADDRESS=`eval 'echo "$ADDRESS_'"$KEY"\"`
+	    INTERFACE=`addrtoif "$ADDRESS"`
+	    DRIVER=`eval 'echo "$WPA_DRIVER_'"$KEY"\"`
+	    IFDELAY=`eval 'echo "$DELAY_'"$KEY"\"`
 	    "$wpa_cli" -g "$CTRL" interface_add "$INTERFACE" "" \
 		"${DRIVER:-wext}" "$CTRLDIR" > /dev/null \
 		| sed -e '/^OK$/d'
-	    NETWORK=`"$wpa_cli" -p "$CTRLDIR" -i "$INTERFACE" add_network`
-	    eval wpa_interface_"$INTERFACE"
+	    NETWORK=`"$wpa_cli" -p "$CTRLDIR" -i "$INTERFACE" \
+		add_network`
+	    eval wpa_interface_"$KEY"
 	    "$wpa_cli" -p "$CTRLDIR" -i "$INTERFACE" enable_network \
 		"$NETWORK" | sed -e '/^OK$/d'
-	    ( sleep "${DELAY:-$DELAY}" || : ) &
+	    sleep "${IFDELAY:-$DELAY}" &
 	    sleep=$!
 	    while :; do
-		kill -0 $sleep || break
-		STATE=`"$wpa_cli" -p "$CTRLDIR" -i "$INTERFACE" status | sed -n -e 's/^wpa_state=//p'`
+		kill -0 $sleep 2>/dev/null || break
+		STATE=`"$wpa_cli" -p "$CTRLDIR" -i "$INTERFACE" \
+		    status | sed -n -e 's/^wpa_state=//p'`
 		if [ "$STATE" = COMPLETED ]; then
-		    kill $sleep
-		    break
+		    while :; do
+			kill -0 $sleep 2>/dev/null || break 2
+			UP=`cat /sys/class/net/"$INTERFACE"/operstate`
+			if [ "$UP" = up ]; then
+			    kill $sleep 2>/dev/null
+			    break 2
+			fi
+			sleep 1
+		    done
 		fi
 		sleep 1
 	    done &
 	    wait $sleep || :
-	    IPADDRS=`eval 'echo "$IPADDRS_'"$INTERFACE"\"`
+	    IPADDRS=`eval 'echo "$IPADDRS_'"$KEY"\"`
 	    if [ -n "$IPADDRS" ]; then
 		if [ "$IPADDRS" = dhcp ]; then
 		    ipconfig -c dhcp -d "$INTERFACE" || :
@@ -99,7 +118,7 @@
 		    done
 		fi
 	    fi
-	    ROUTES=`eval 'echo "$ROUTES_'"$INTERFACE"\"`
+	    ROUTES=`eval 'echo "$ROUTES_'"$KEY"\"`
 	    if [ -n "$ROUTES" ]; then
 		for route in $ROUTES; do
 		    "$ip" route add "$route" dev "$BRIDGE"
@@ -109,7 +128,9 @@
 	;;
     stop)
 	"$wpa_cli" -g "$CTRL" terminate 2>&1 | sed -e '/^OK$/d'
-        for INTERFACE in $interfaces; do
+        for KEY in $ifkeys; do
+	    ADDRESS=`eval 'echo "$ADDRESS_'"$KEY"\"`
+	    INTERFACE=`addrtoif "$ADDRESS"`
 	    "$ip" addr show scope global permanent dev "$INTERFACE" \
 		| while read type addr rest; do
 		case "$type" in
@@ -130,7 +151,7 @@
 	if [ "$IPADDRS" = dhcp ]; then
 	    echo af_packet
 	fi
-	sed -n -e 's/#.*$//' -e 's/[ 	]*$//' -e 's/^MODULE=//p' \
-	    "$CONFIG"
+	sed -n -e 's/#.*$//' -e 's/[ 	]*$//' \
+	    -e 's/^MODULE_[^=]\+=//p' "$CONFIG"
 	;;
 esac

=== modified file 'network-hooks.d/wireless.conf'
--- network-hooks.d/wireless.conf	2011-12-27 03:56:39 +0000
+++ network-hooks.d/wireless.conf	2011-12-31 13:25:58 +0000
@@ -2,16 +2,22 @@
 #WPAS_OPTIONS=""
 
 # wlan0
-MODULE_wlan0=ath9k
-#WPA_DRIVER_wlan0=wext
-wpa_interface_wlan0(){
+ADDRESS_0=00:11:22:33:44:55
+MODULE_0=ath9k
+#WPA_DRIVER_0=wext
+wpa_interface_0(){
     # Use this format to set simple things:
     wpa_cli_set ssid home
     wpa_cli_set psk "secret passphrase"
     # Use this format to do more complex things with wpa_cli:
-    #"$wpa_cli" -p "$CTRLDIR" -i "$INTERFACE" set_network "$NETWORK" passphrase '"secret passphrase"'
+    #"$wpa_cli" -p "$CTRLDIR" -i "$INTERFACE" bssid "$NETWORK" 00:11:22:33:44:55
+    #"$wpa_cli" -g "$CTRL" ping
 }
-#DELAY_wlan0=10
-IPADDRS_wlan0=dhcp
-#IPADDRS_wlan0="192.0.2.3/24 2001:DB8::aede:48ff:fe71:f6f2/32"
-#ROUTES_wlan0="192.0.2.0/24 2001:DB8::/32"
+#DELAY_0=10
+IPADDRS_0=dhcp
+#IPADDRS_0="192.0.2.3/24 2001:DB8::aede:48ff:fe71:f6f2/32"
+#ROUTES_0="192.0.2.0/24 2001:DB8::/32"
+
+#ADDRESS_1=11:22:33:44:55:66
+#MODULE_1=...
+#...

=== modified file 'plugins.d/mandos-client.c'
--- plugins.d/mandos-client.c	2011-12-30 17:14:15 +0000
+++ plugins.d/mandos-client.c	2011-12-30 18:48:49 +0000
@@ -1474,7 +1474,7 @@
 	}
 	free(delaystring);
 	if(connect_to != NULL){
-	  ret = setenv("CONNECT", connect_to);
+	  ret = setenv("CONNECT", connect_to, 1);
 	  if(ret == -1){
 	    perror_plus("setenv");
 	    _exit(EX_OSERR);

